package com.blackberry.workspaces.sdk.api.examples;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.blackberry.workspaces.sdk.api.APISession;
import com.blackberry.workspaces.sdk.api.enums.EntityType;
import com.blackberry.workspaces.sdk.api.enums.YesNoDefault;
import com.blackberry.workspaces.sdk.api.json.AddEntityVdrJson;
import com.blackberry.workspaces.sdk.api.json.AddMemberToGroupJson;
import com.blackberry.workspaces.sdk.api.json.AddMembersToGroupWithGroupJson;
import com.blackberry.workspaces.sdk.api.json.AddRoomMemberJson;
import com.blackberry.workspaces.sdk.api.json.AddRoomMembersJson;
import com.blackberry.workspaces.sdk.api.json.BaseJson;
import com.blackberry.workspaces.sdk.api.json.BulkOperationResultJson;
import com.blackberry.workspaces.sdk.api.json.CreateRoomJson;
import com.blackberry.workspaces.sdk.api.json.DeleteRoomJson;
import com.blackberry.workspaces.sdk.api.json.DocumentJson;
import com.blackberry.workspaces.sdk.api.json.EditRoomJson;
import com.blackberry.workspaces.sdk.api.json.FolderJson;
import com.blackberry.workspaces.sdk.api.json.InviteJson;
import com.blackberry.workspaces.sdk.api.json.ItemListJson;
import com.blackberry.workspaces.sdk.api.json.ListDocumentsVdrJson;
import com.blackberry.workspaces.sdk.api.json.MoveFolderJson;
import com.blackberry.workspaces.sdk.api.json.PagingItemListJson;
import com.blackberry.workspaces.sdk.api.json.PermissionFromUserJson;
import com.blackberry.workspaces.sdk.api.json.PermissionSetJson;
import com.blackberry.workspaces.sdk.api.json.PermittedEntityFromUserJson;
import com.blackberry.workspaces.sdk.api.json.RenameFolderJson;
import com.blackberry.workspaces.sdk.api.json.RoomJson;
import com.blackberry.workspaces.sdk.api.json.RoomLinksJson;
import com.blackberry.workspaces.sdk.api.json.VdrAddPermissionsJson;
import com.blackberry.workspaces.sdk.api.json.VdrEditPermissionsJson;
import com.blackberry.workspaces.sdk.api.json.WorkspaceInfoJson;
import com.blackberry.workspaces.sdk.api.resource.Workspaces;

/**
 * Examples of Workspaces Workspace functionality
 * 
 * @author tasfaw
 *
 */
public class WorkspacesExample
{
    private APISession apiSession;

    /**
     * Creates an instances of the WorkspacesExample class
     * @param apiSession An ApiSession instance to be used.
     */
    public WorkspacesExample(APISession apiSession)
    {
        this.apiSession = apiSession;
    }

  /*
   * Example to create a room (workspace).
   */
  public RoomJson createRoom(String name, String description, List<String> administrators)
  {
      // START SNIPPET: devGuide1

    Workspaces workspaces = apiSession.getWorkspacesResource();

    // Create the JSON object needed for the method call, and set its values.
    CreateRoomJson createRoomJson = new CreateRoomJson();
    createRoomJson.setName(name);
    createRoomJson.setDescription(description);
    createRoomJson.setAdministrators(administrators);

    // Using the auth token from Example 1, call the method, and get a JSON object back
    RoomJson roomJson = workspaces.createRoomV30(createRoomJson);

    // END SNIPPET: devGuide1
    return roomJson;
  }

  /*
   * Creates a group and adds it to the room
   */
  public String addGroupToRoom(String workspaceId, String groupName)
  {
      //START SNIPPET: devGuide2

     Workspaces workspaces = apiSession.getWorkspacesResource();

    // Create a JSON object the represents the new group and set its values
    PermittedEntityFromUserJson permittedEntityFromUserJson =
        new PermittedEntityFromUserJson();
    permittedEntityFromUserJson.setAddress(groupName);
    permittedEntityFromUserJson.setEntityType(EntityType.GROUP);

    // The JSON object needed for the resource method call
    AddEntityVdrJson addEntityVdrJson = new AddEntityVdrJson();

    // Create a new permissions JSON with default values
    PermissionFromUserJson permissionFromUserJson = new PermissionFromUserJson();

    addEntityVdrJson.setPermittedEntity(permittedEntityFromUserJson);
    addEntityVdrJson.setNewPermissions(permissionFromUserJson);

    // Make the call to the Rooms resource and return "success"
    String result = workspaces.addEntityV30(workspaceId, addEntityVdrJson);

    // END SNIPPET: devGuide2
    return result;
  }

  public String addUsersToGroup(List<String> userAddresses, String groupName, int roomId)
  {
      //START SNIPPET: devGuide3

    Workspaces workspaces = apiSession.getWorkspacesResource();

    List<AddMemberToGroupJson> memberList = new ArrayList<AddMemberToGroupJson>();

    // Loop through the List<String> userAddresses & make a AddMemberToGroupJson for each user
    for (String currentAddress : userAddresses)
    {
      PermittedEntityFromUserJson currentEntity = new PermittedEntityFromUserJson();
      currentEntity.setAddress(currentAddress);
      currentEntity.setEntityType(EntityType.USER);

      AddMemberToGroupJson currentMemberJson = new AddMemberToGroupJson();
      currentMemberJson.setEntity(currentEntity);

      memberList.add(currentMemberJson);
    }

    AddMembersToGroupWithGroupJson groupMemberJson = new AddMembersToGroupWithGroupJson();
    groupMemberJson.setMembersList(memberList);

    // roomId is the integer identifying the room
    groupMemberJson.setRoomId(roomId);

    // Set the group name to be a string
    groupMemberJson.setGroupName(groupName);

    // Make the call to the Rooms resource and return "success"
    String result = workspaces.addMembersToGroupV30(groupMemberJson);

 // END SNIPPET: devGuide3
    return (result);
  }

  /*
   * Example to delete a room (workspace).
   */
  public BulkOperationResultJson deleteRoom(boolean isPermanent, int roomId)
  {
    Workspaces workspaces = apiSession.getWorkspacesResource();
    DeleteRoomJson deleteRoomJson = new DeleteRoomJson();

    deleteRoomJson.setIsPermanent(isPermanent);
    deleteRoomJson.setRoomId(roomId);

    BulkOperationResultJson result = workspaces.deleteRoomV30(deleteRoomJson);

    return result;
  }

  /*
   * Example to edit a room (workspace).
   */
  public RoomJson editRoom(String roomId, String name, String description,
      boolean renameIfExists, boolean isPersonal, boolean shouldUpdatePersonal)
  {
    Workspaces workspaces = apiSession.getWorkspacesResource();
    EditRoomJson editRoomJson = new EditRoomJson();

    editRoomJson.setName(name);
    editRoomJson.setDescription(description);
    editRoomJson.setRenameIfExists(renameIfExists);
    editRoomJson.setIsPersonal(isPersonal);
    editRoomJson.setShouldUpdatePersonal(shouldUpdatePersonal);

    RoomJson room = workspaces.editRoomV30(roomId, editRoomJson);

    return room;
  }

  /*
   * Example to get a room (workspace) info.
   */
  public WorkspaceInfoJson infoRoom(String roomId)
  {
    Workspaces workspaces = apiSession.getWorkspacesResource();

    return workspaces.getInfoV30(roomId, true, true, false, false, false);
  }

  /*
   * Example to list all the virtual data rooms of the current user.
   */
  public ItemListJson<WorkspaceInfoJson> listRooms()
  {
      // START SNIPPET: devGuide11

    Workspaces workspaces = apiSession.getWorkspacesResource();

    // This returns a list of rooms, which can be iterated over. The other parameters include:
    // addExternalData, adminMode, includeSyncData, includeWorkspacePolicyData, and
    // workspaceTypes. Please see the javadoc documentation for details.
    ItemListJson<WorkspaceInfoJson> itemListJson =
        workspaces.listRoomsV30(true, true, false, false, null);

    // END SNIPPET: devGuide11
    return itemListJson;
  }

  /*
   * Example to gets the tree of all folders and subfolders in a specified workspace.
   */
  public FolderJson getFolderTree(String roomId)
  {
    Workspaces workspaces = apiSession.getWorkspacesResource();
    FolderJson folder = workspaces.getFolderTreeV30(roomId, null);

    return folder;
  }

  /*
   * Example to gets subfolders in a specified workspace.
   */
  public List<FolderJson> getSubFolders(String roomId)
  {
      // START SNIPPET: devGuide12

    Workspaces workspaces = apiSession.getWorkspacesResource();

    // This returns a folder object, which contains details about the current workspace, as
    // well as a sub folder list that can be iterated over.
    FolderJson folderJson = workspaces.getFolderTreeV30(roomId, null);

    List<FolderJson> subFolders = folderJson.getSubFolders();

    // END SNIPPET: devGuide12
    return subFolders;
  }

  /*
   * Example to get URL link to a workspace.
   */
  public RoomLinksJson getRoomLinks(String roomId)
  {
    Workspaces workspaces = apiSession.getWorkspacesResource();
    RoomLinksJson roomLinks = workspaces.getRoomLinksV30(roomId);

    return roomLinks;
  }

  /*
   * Example to rename a folder within a workspace using folderGUID(it can also be done using either folderId or path)
   */
  public String renameFolder(String roomId, String folderGuid, String newFolderName)
  {
    Workspaces workspaces = apiSession.getWorkspacesResource();
    RenameFolderJson renameFolderJson = new RenameFolderJson();

    renameFolderJson.setFolderGuid(folderGuid);
    renameFolderJson.setNewFolderName(newFolderName);

    String result = workspaces.renameFolderV30(roomId, renameFolderJson);

    return result;
  }

  /*
   * Example to move folders in a room (workspace).
   */
  public BulkOperationResultJson moveFolders(String roomId, String currentPath,
      String newPath)
  {
    Workspaces workspaces = apiSession.getWorkspacesResource();
    MoveFolderJson moveFolderJson = new MoveFolderJson();

    moveFolderJson.setCurrentPath(currentPath);
    moveFolderJson.setNewPath(newPath);

    BulkOperationResultJson result =
        workspaces.moveFolderV30(roomId, moveFolderJson);

    return result;
  }

  /*
   * Example to invite groups to a room (workspace).
   */
  public BulkOperationResultJson inviteGroup(String roomId, String emailSubject,
      String emailMessage, Set<String> groupNames, Set<String> additionalEmails)
  {
    Workspaces workspaces = apiSession.getWorkspacesResource();
    InviteJson inviteJson = new InviteJson();

    inviteJson.setEmailSubject(emailSubject);
    inviteJson.setEmailMessage(emailMessage);
    inviteJson.setGroupNames(groupNames);
    inviteJson.setAdditionalEmails(additionalEmails);

    BulkOperationResultJson result = workspaces.inviteV30(roomId, inviteJson);

    return result;
  }

  /*
   * Example to add members to a room (workspace).
   */
  public String addMembers(String roomId, String address, EntityType entityType,
      boolean isUsersDefaultEntity, boolean downloadOriginal, String usersRole)
  {
    Workspaces workspaces = apiSession.getWorkspacesResource();
    AddRoomMembersJson addRoomMembersJson = new AddRoomMembersJson();
    PermissionFromUserJson permissionFromUserJson = new PermissionFromUserJson();
    PermittedEntityFromUserJson permittedEntityFromUserJson =
        new PermittedEntityFromUserJson();
    AddRoomMemberJson addRoomMemberJson = new AddRoomMemberJson();

    List<AddRoomMemberJson> addRoomMemberJsonList = new ArrayList<AddRoomMemberJson>();
    addRoomMemberJsonList.add(addRoomMemberJson);

    permissionFromUserJson.setDownloadOriginal(downloadOriginal);
    permittedEntityFromUserJson.setAddress(address);
    permittedEntityFromUserJson.setEntityType(entityType);

    addRoomMemberJson.setPermittedEntity(permittedEntityFromUserJson);

    addRoomMembersJson.setNewMembers(addRoomMemberJsonList);
    addRoomMembersJson.setIsUsersDefaultEntity(isUsersDefaultEntity);
    addRoomMembersJson.setUsersPermissions(permissionFromUserJson);
    addRoomMembersJson.setUsersRole(usersRole);

    String result = workspaces.addMembersV30(roomId, addRoomMembersJson);

    return result;
  }

  public BulkOperationResultJson addGroupAccessToDocument(String roomId,
      String documentName, String groupName)
  {
      // START SNIPPET: devGuide4

    Workspaces workspaces = apiSession.getWorkspacesResource();

    // Create a json object for the document search, and accept its defaults.
    // Adjust if needed.
    ListDocumentsVdrJson listDocumentsJson = new ListDocumentsVdrJson();

    // Make the call to get the list of documents
    PagingItemListJson<BaseJson> documentList =
        workspaces.listDocumentsV30(roomId, listDocumentsJson);

    // END SNIPPET: devGuide4
    // Assume there's only 1 document and just get(0). If there are more,
    // then iterate the List and find the needed one by its name.
    DocumentJson myDoc = (DocumentJson) documentList.getItems().get(0);

    List<PermittedEntityFromUserJson> permissionsList =
        new ArrayList<PermittedEntityFromUserJson>();

    // START SNIPPET: devGuide5

    PermittedEntityFromUserJson groupPermission = new PermittedEntityFromUserJson();
    groupPermission.setAddress(groupName);
    groupPermission.setEntityType(EntityType.GROUP);

    permissionsList.add(groupPermission);

    // Adjust permissions as needed
    PermissionSetJson permissionSet = new PermissionSetJson();
    permissionSet.setDownloadOriginal(YesNoDefault.YES);
    permissionSet.setDownloadControlled(YesNoDefault.YES);
    permissionSet.setCopy(YesNoDefault.YES);
    permissionSet.setEdit(YesNoDefault.YES);
    permissionSet.setPrint(YesNoDefault.YES);
    permissionSet.setProgrammaticAccess(YesNoDefault.YES);
    permissionSet.setSpotlight(YesNoDefault.YES);
    permissionSet.setWatermark(YesNoDefault.YES);

    VdrAddPermissionsJson permissionsJson = new VdrAddPermissionsJson();
    permissionsJson.setPermittedEntities(permissionsList);
    permissionsJson.setPermissionSet(permissionSet);

    Set<String> documentGuids = new HashSet<String>();

    documentGuids.add(myDoc.getGuid());
    permissionsJson.setDocumentGuids(documentGuids);

    // END SNIPPET: devGuide5

    // START SNIPPET: devGuide6

    BulkOperationResultJson result =
        workspaces.addPermissionsV30(roomId, permissionsJson);

    // END SNIPPET: devGuide6
    return result;
  }

  public BulkOperationResultJson editGroupAccessToDocument(String roomId,
      String documentName, String groupName)
  {
    Workspaces workspaces = apiSession.getWorkspacesResource();

    // Create a json object for the document search, and accept its defaults. Adjust if needed.
    ListDocumentsVdrJson listDocumentsJson = new ListDocumentsVdrJson();

    // Make the call to get the list of documents
    PagingItemListJson<BaseJson> documentList =
        workspaces.listDocumentsV30(roomId, listDocumentsJson);

    // Assume there's only 1 document and just get(0). If there are more, then iterate the List
    // and find the needed one by its name.
    DocumentJson myDoc = (DocumentJson) documentList.getItems().get(0);

    // START SNIPPET: devGuide10

    // Create the json that indicates the group that has permissions to the document
    PermittedEntityFromUserJson groupPermission = new PermittedEntityFromUserJson();
    groupPermission.setAddress(groupName);
    groupPermission.setEntityType(EntityType.GROUP);

    List<PermittedEntityFromUserJson> permissionsList =
        new ArrayList<PermittedEntityFromUserJson>();
    permissionsList.add(groupPermission);

    // Add the document GUID
    Set<String> documentGuids = new HashSet<String>();
    documentGuids.add(myDoc.getGuid());

    // Create the json for the permissions and set them to NO
    PermissionSetJson permissionSet = new PermissionSetJson();
    permissionSet.setEdit(YesNoDefault.NO);
    permissionSet.setPrint(YesNoDefault.NO);
    permissionSet.setSpotlight(YesNoDefault.NO);
    permissionSet.setWatermark(YesNoDefault.NO);

    VdrEditPermissionsJson editPermissionsJson = new VdrEditPermissionsJson();
    editPermissionsJson.setPermittedEntities(permissionsList);
    editPermissionsJson.setDocumentGuids(documentGuids);
    editPermissionsJson.setPermissionSet(permissionSet);

    // Make the call and get a BulkOperationResultJson back
    BulkOperationResultJson result =
        workspaces.editPermissionsV30(roomId, editPermissionsJson);

    // END SNIPPET: devGuide10
    return result;
  }
}
